@@ -4,12 +4,28 @@ use strict;
 use warnings;
 
 use CGI;
+use CGI::Application;
 use List::MoreUtils qw(uniq);
 use URI::Escape;
 
-use base qw(CGI::Application::Dispatch);
+use base qw/CGI::Application::Dispatch Exporter/;
 
-our $VERSION = 0.02;
+our %EXPORT_TAGS = (
+    constants => [
+        qw/URI_IS_PERMALINK URI_UNKNOWN_PARAM URI_PATH_PARAM_IN_QUERY/,
+        qw/URI_PARAM_NOT_IN_ORDER URI_OMITTABLE_PARAM/,
+    ],
+);
+$EXPORT_TAGS{all} = [ uniq map { @$_ } values %EXPORT_TAGS ];
+our @EXPORT_OK = @{$EXPORT_TAGS{all}};
+
+our $VERSION = 0.03;
+
+use constant URI_IS_PERMALINK        => 0;
+use constant URI_UNKNOWN_PARAM       => 1;
+use constant URI_PATH_PARAM_IN_QUERY => 2;
+use constant URI_PARAM_NOT_IN_ORDER  => 3;
+use constant URI_OMITTABLE_PARAM     => 4;
 
 our %dispatch_table;
 our %uri_table;
@@ -28,7 +44,7 @@ sub register {
         my $app = $entry->{app} || (caller)[0];
         my $host = $entry->{host} || '*';
         my $proto = $entry->{protocol} || 'http';
-        my $build_uri = $entry->{build_uri};
+        my $uri_table_entry;
         my $rm;
         unless ($rm = $entry->{rm}) {
             unless (ref $entry->{path}) {
@@ -49,7 +65,8 @@ sub register {
                 my $proto2 = delete $action->{protocol} || $proto;
                 $dispatch_table->{$host} ||= [];
                 push @{$dispatch_table->{$host2}}, $path, $action;
-                $build_uri ||= _build_default_uri_builder({
+                $uri_table_entry ||= _build_uri_table_entry({
+                    %$entry,
                     protocol => $proto2,
                     host     => $host2,
                     path     => $path,
@@ -64,7 +81,8 @@ sub register {
             };
             $dispatch_table->{$host} ||= [];
             push @{$dispatch_table->{$host}}, $entry->{path}, $action;
-            $build_uri ||= _build_default_uri_builder({
+            $uri_table_entry ||= _build_uri_table_entry({
+                %$entry,
                 protocol => $proto,
                 host     => $host,
                 path     => $entry->{path},
@@ -72,87 +90,27 @@ sub register {
                 action   => $action,
             });
         }
-        $uri_table->{"$app/$rm"} = $build_uri;
+        $uri_table_entry->{build_uri} = $entry->{build_uri} || undef;
+        $uri_table->{"$app/$rm"} = $uri_table_entry;
         unless ($app_init_map{$app}) {
             $app_init_map{$app} = 1;
             no strict 'refs';
+            my $self_klass = ref($self) || $self;
             $app->add_callback(
-                'init',
+                'prerun',
                 sub {
                     my $app = shift;
-                    setup_runmodes($app, ref($self) || $self);
+                    _setup_runmodes($app, $self_klass);
                 },
             );
-            *{"${app}::build_uri"} = sub {
-                _app_build_uri($_[0], ref($self) || $self, $_[1]);
+            *{"${app}::_uri_mapping"} = sub {
+                _uri_mapping_of($self_klass, $app, $_[1]);
             };
+            *{"${app}::all_param"} = \&all_param;
         }
     };
 }
 
-sub build_uri {
-    my ($self, $args) = @_;
-    
-    my $app = $args->{app}
-        or die "no 'app'\n";
-    my $rm = $args->{rm} || undef;
-    unless ($rm) {
-        $app =~ m|[^:]*$|;
-        $rm = lcfirst $&;
-        $rm =~ s/[A-Z]/'_' . lc($&)/ego;
-    }
-    my $params = $args->{params} || [];
-    
-    my $uri_table = ($uri_table{ref($self) || $self} ||= {});
-    
-    die "no data for $app/$rm" unless exists $uri_table->{"$app/$rm"};
-    $uri_table->{"$app/$rm"}->(
-        $args->{protocol} || undef,
-        sub {
-            my $n = shift;
-            foreach my $h (@$params) {
-                if (ref $h eq 'HASH') {
-                    return ($h->{$n}) if exists $h->{$n};
-                } else {
-                    my @v = $h->param($n);
-                    return @v if @v;
-                }
-            }
-            ();
-        });
-}
-
-sub _app_build_uri {
-    my ($app, $mapping, $args) = @_;
-    
-    $args = { params => $args }
-        if ref($args) eq 'ARRAY';
-    
-    build_uri(
-        $mapping,
-        {
-            app => ref $app || $app,
-            $args ? %$args : (),
-        },
-    );
-}
-
-sub run_modes_of {
-    my ($self, $app) = @_;
-    my $dispatch_table = ($dispatch_table{ref($self) || $self} ||= []);
-    
-    $dispatch_table = $dispatch_table->{CGI::virtual_host()}
-        || $dispatch_table{'*'};
-    
-    my @rm = uniq map {
-        $_->{rm}
-    } grep {
-        ref($_) && $_->{app} eq $app
-    } @$dispatch_table;
-    
-    \@rm;
-}
-
 sub dispatch_args {
     my $self = shift;
     my $dispatch_table = ($dispatch_table{ref($self) || $self} ||= {});
@@ -165,41 +123,130 @@ sub dispatch_args {
     };
 }
 
-sub setup_runmodes {
-    my ($app, $mapping) = @_;
+sub all_param {
+    my $app = shift;
     
-    my %cur = $app->run_modes();
+    if (@_ == 1) {
+        my $n = shift;
+        my $v = $app->param($n);
+        return $v
+            if defined $v;
+        return $app->query->param($n);
+    }
     
-    $app->run_modes([
-        grep {
-            ! exists $cur{$_}
-        } @{run_modes_of($mapping, ref $app)},
-    ]);
+    $app->param(@_);
 }
 
-sub _build_default_uri_builder {
-    my $prototype = shift;
+*CGI::Application::uri_mapping = sub {
+    my $app = shift;
+    my $mapping;
     
-    $prototype->{path} =~ s|^/?(.*)/?$|$1|;
-    $prototype->{path} = [
-        split '/', $prototype->{path}
-    ];
-    $prototype->{query} ||= [];
-    
-    sub {
-        _default_build_uri($prototype, @_);
+    eval {
+        $mapping = $app->_uri_mapping(@_);
     };
+    die "no mapping for $app, did you register the class?\n"
+        unless $mapping;
+    
+    $mapping;
+};
+
+*CGI::Application::build_uri = sub {
+    my ($app, $args) = @_;
+    my $rm = $args->{rm} || undef
+        if ref($args) eq 'HASH';
+    
+    _build_uri($app->uri_mapping($rm), $args);
+};
+
+*CGI::Application::validate_uri = sub {
+    my ($app, $args) = @_;
+    my $mapping = $app->uri_mapping($args->{rm} || undef);
+    
+    return _validate_uri($mapping, $app, $args->{extra} || []);
+};
+
+*CGI::Application::normalize_uri = sub {
+    my ($app, $args) = @_;
+    my $mapping = $app->uri_mapping($args->{rm} || undef);
+    
+    return
+        if $app->validate_uri($mapping, $app, $args->{extra} || []);
+    
+    return $app->redirect(_build_uri(
+        $mapping,
+        {
+            rm     => $args->{rm} || undef,
+            params => [
+                $app,
+                $app->query,
+            ],
+        },
+    ));
+};
+
+sub _run_modes_of {
+    my ($self, $app) = @_;
+    my $dispatch_table = ($dispatch_table{ref($self) || $self} ||= []);
+    
+    $dispatch_table = $dispatch_table->{CGI::virtual_host()}
+        || $dispatch_table->{'*'};
+    
+    my @rm = uniq map {
+        $_->{rm}
+    } grep {
+        ref($_) && $_->{app} eq $app
+    } @$dispatch_table;
+    
+    \@rm;
+}
+
+sub _uri_mapping_of {
+    my ($self, $app, $rm) = @_;
+    
+    $rm ||= _pkg2rm($app);
+    
+    my $mapping = ($uri_table{ref($self) || $self} ||= {})->{"$app/$rm"}
+        or die "mapping for $app/$rm not found, did you register $app?\n";
+    
+    $mapping;
+}
+
+sub _build_uri {
+    my ($prototype, $args) = @_;
+    
+    $args = { params => $args }
+        if ref($args) eq 'ARRAY';
+    my $params = $args->{params} || [];
+    
+    ($prototype->{build_uri} || \&_default_build_uri)->(
+        {
+            %$prototype,
+            protocol => $args->{protocol} || $prototype->{protocol},
+        },
+        sub {
+            my $n = shift;
+            foreach my $h (@$params) {
+                if (ref $h eq 'HASH') {
+                    return ($h->{$n}) if exists $h->{$n};
+                } else {
+                    my @v = $h->param($n);
+                    return wantarray ? @v : $v[0]
+                        if @v;
+                }
+            }
+            ();
+        });
 }
 
 sub _default_build_uri {
-    my ($prototype, $protocol, $get_param) = @_;
+    my ($prototype, $get_param) = @_;
     
     # determine hostport
     my $host = $prototype->{host};
     $host = CGI::virtual_host() if $host eq '*';
     # build path
     my @path;
-    foreach my $p (@{$prototype->{path}}) {
+    foreach my $p (@{$prototype->{path_array}}) {
         if ($p =~ m|^:(.*?)(\??)$|) {
             my ($n, $optional) = ($1, $2);
             my @v = $get_param->($n);
@@ -217,18 +264,110 @@ sub _default_build_uri {
     }
     # build query params
     my @qp;
-    foreach my $n (@{$prototype->{query}}) {
-        my @v = $get_param->($n);
-        push @qp, map { "$n=" . uri_escape($_) } @v;
+    foreach my $p (@{$prototype->{query}}) {
+        my @v = $get_param->($p->{name});
+        foreach my $v (@v) {
+            if ($p->{omit}) {
+                next if $v eq $p->{omit};
+            }
+            push @qp, "$p->{name}=" . uri_escape($v);
+        }
     }
     # build and return
-    my $uri = ($protocol || $prototype->{protocol})
-        . "://$host/" . join('/', @path);
+    my $uri = "$prototype->{protocol}://$host/" . join('/', @path);
     $uri .= '?' . join('&', @qp)
         if @qp;
     $uri;
 }
 
+sub _validate_uri {
+    my ($mapping, $app, $extra) = @_;
+    my $param_map = $mapping->{param_map};
+    my $query = $app->query;
+    my $meth = $query->request_method || 'GET';
+    $extra = { map { $_ => 1 } @$extra };
+    
+    return URI_IS_PERMALINK
+        unless $meth eq 'GET' || $meth eq 'HEAD';
+    
+    my $max_rank = 0;
+    foreach my $n (
+        map { (split '=', $_, 2)[0] } split(/[&;]/, $query->query_string)
+    ) {
+        if (my $ref = $param_map->{$n}) {
+            return URI_PATH_PARAM_IN_QUERY
+                if $ref->{rank} < 0;
+            return URI_PARAM_NOT_IN_ORDER
+                if $ref->{rank} < $max_rank;
+            if (my $omit = $ref->{omit}) {
+                foreach my $v ($query->param($n)) {
+                    return URI_OMITTABLE_PARAM
+                        if $v eq $omit;
+                }
+            }
+            $max_rank = $ref->{rank};
+        } else {
+            return URI_UNKNOWN_PARAM
+                unless $extra->{$n};
+        }
+    }
+    
+    URI_IS_PERMALINK;
+}
+
+sub _setup_runmodes {
+    my ($app, $mapping) = @_;
+    $app->run_modes(_run_modes_of($mapping, ref $app));
+}
+
+sub _build_uri_table_entry {
+    my $table = shift;
+    
+    # setup path_array
+    my $p = $table->{path};
+    $p =~ s|^/?(.*)/?$|$1|;
+    $table->{path_array} = [ split '/', $p ];
+    
+    # normalize query array
+    $table->{query} ||= [];
+    foreach my $p (@{$table->{query}}) {
+        $p = {
+            name => $p,
+        } unless ref $p;
+    }
+    
+    # setup param_map
+    $table->{param_map} = {};
+    for (my $i = 0; $i < @{$table->{query}}; $i++) {
+        $table->{param_map}->{$table->{query}->[$i]->{name}} = {
+            rank => $i + 1,
+            omit => $table->{query}->[$i]->{omit} || undef,
+        };
+    }
+    foreach my $e (@{$table->{path_array}}) {
+        if ($e =~ /^:(.*?)\??$/) {
+            $table->{param_map}->{$1} = {
+                rank => -1,
+            };
+        }
+    }
+    
+    # set build_uri
+    $table->{build_uri} ||= \&_default_build_uri;
+    
+    $table;
+}
+
+sub _pkg2rm {
+    my $pkg = shift;
+    
+    $pkg =~ m|[^:]*$|;
+    my $rm = $&;
+    $rm =~ s/([a-z]?)([A-Z])/($1 ? "$1_" : '') . lc($2)/ego;
+    
+    $rm;
+}
+
 1;
 
 __END__
@@ -239,12 +378,19 @@ CGI::Application::URIMapping - A dispatcher and permalink builder
 
 =head1 SYNOPSIS
 
+  # your.cgi
+  use MyApp::URIMapping;
+  
+  MyApp::URIMapping->dispatch();
+  
+  
   package MyApp::URIMapping;
   
   use base qw/CGI::Application::URIMapping/;
   use MyApp::Page1;
   use MyApp::Page2;
   
+  
   package MyApp::Page1;
   
   use base qw/CGI::Application/;
@@ -256,6 +402,13 @@ CGI::Application::URIMapping - A dispatcher and permalink builder
   });
   
   sub page1 {
+    my $self = shift;
+    
+    # if URI is not in permalink style, redirect
+    if (my $r = $self->normalize_uri) {
+      return $r;
+    }
+    
     ...
   }
   
@@ -332,6 +485,16 @@ Limits the registration to given host if specified.
 
 List of parameters to be marshallised when building a premalink.  The parameters will be marshallized in the order of the array.
 
+=head2 all_param
+
+The function is an accessor / mutator for uniformly handling path parameters and query parameters.
+
+  my $value = $cgi_app->all_param($name);   # read paramater
+  
+  $cgi_app->all_param($name, $value);       # set parameter
+
+The setter first tries to read from $cgi_app->param($name), and then $cgi_app->query->param($name).  The getter sets the value to $cgi_app->param($name, $value).
+
 =head2 build_uri
 
 The function is automatically setup for the registered CGI::Application packages.
@@ -374,6 +537,52 @@ Protocol.
 
 An array of hashes or object containing values to be filled in when building the URI.  The parameters are searched from the begging of the array to the end, and the first-found value is used.  If the array element is an object, its C<param> method is called in order to obtain the variable.
 
+=head2 validate_uri
+
+The function, which is automaticaly setup as a instance method of CGI::Application, checks whether the current URI is conforms to the registered format.
+
+  $cgi_app->validate_uri();
+  
+  $cgi_app->validate_uri({        # explicitly specify runmode
+     rm => 'page1',
+  };
+  
+  $cgi_app->validate_uri({        # set extra query parameters to be allowed
+     extra => [ qw/__extra1 __extra2/ ],
+  });
+
+The function accepts following attributes.
+
+=head3 rm
+
+Runmode.  If omitted, uncamelized basename of the package is used.
+
+=head3 extra
+
+Array of query args to be ignored while validating the parameters received.
+
+The return value of the function is one of the following constants.
+
+=head3 URI_IS_PERMALINK
+
+Current URI conforms the registered format.
+
+=head3 URI_UNKNOWN_PARAM
+
+Current URI contains an unknown query parameter.
+
+=head3 URI_PATH_PARAM_IN_QUERY
+
+A parameter expected in path_info is being received as a query parameter.
+
+=head3 URI_OMITTABLE_PARAM
+
+A parameter that should be omitted (since it contains the default value) exists.
+
+The constants are importable by specifying C<:constants> attribute.
+
+  use CGI::Application::URIMapping qw/:constants/;
+
 =head1 AUTHOR
 
 Copyright (c) 2007 Cybozu Labs, Inc.  All rights reserved.
